"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function mapObjectToArray(obj, keyName) {
    const mappedDatas = [];
    for (const key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            mappedDatas.push({ ...obj[key], [keyName]: key });
        }
    }
    return mappedDatas;
}
// This code repetition is in favor of semantics
const isRealtimeDBAutoGeneratedKey = (key) => (typeof key === 'string' &&
    key[0] === '-' &&
    key[5] === '_' &&
    key.length == 20);
//TODO: Test the new variation
const isCustomUIDKey = (key) => (typeof key === 'string' &&
    key[0] === '-' &&
    key[5] === '_' &&
    key.length == 30);
function isFirebaseValidKey(text) {
    return isRealtimeDBAutoGeneratedKey(text) || isCustomUIDKey(text);
}
function listProperties(obj) {
    return Object.getOwnPropertyNames(obj);
}
function hasAllFirebaseKeysInside(variable) {
    return listProperties(variable).every(isFirebaseValidKey);
}
/**
 * Flattens the format recieved from the Firebase Realtime Database
 * @constructor
 * @param {JSON|Array.<Object>} data - The data that's attempt to format, it should be extracted from Firebase's ```DataSnapshot``` .
 * @param {string} [key='id'] - Name of the new attribute that contains the entities's ids.
 */
function firebaseDbFormatter(data, key = 'id') {
    if (Array.isArray(data)) {
        return data.map(obj => firebaseDbObjectFormatter(obj, key));
    }
    else {
        return firebaseDbObjectFormatter(data, key);
    }
}
exports.default = firebaseDbFormatter;
function firebaseDbObjectFormatter(obj, key = 'id') {
    const isAnIterableProperty = (property) => hasAllFirebaseKeysInside(obj[property]) && property in obj;
    const isAnIterablePropertyCustom = (obj, property) => hasAllFirebaseKeysInside(obj[property]) && property in obj;
    const hasIterablePropertiesCustom = (element) => listProperties(element).some((property) => isAnIterablePropertyCustom(element, property));
    const hasIterableProperties = (element) => listProperties(element).some((property) => isAnIterableProperty(property));
    const isAListOfIterables = (element) => listProperties(element).every(property => isFirebaseValidKey(property));
    if (hasIterableProperties(obj)) {
        const properties = listProperties(obj);
        properties.map(property => {
            if (isAnIterableProperty(property)) {
                obj[property] = mapObjectToArray(obj[property], key);
                obj[property].forEach((nestedObj) => {
                    firebaseDbFormatter(nestedObj, key);
                });
            }
        });
    }
    //TODO: Fix this absolute piece of garbage
    if (isAListOfIterables(obj)) {
        const listOfIds = listProperties(obj);
        listOfIds.map(id => {
            const insideObj = obj[id];
            if (hasIterablePropertiesCustom(insideObj)) {
                const properties = listProperties(insideObj);
                properties.map(property => {
                    if (isAnIterablePropertyCustom(insideObj, property)) {
                        insideObj[property] = mapObjectToArray(insideObj[property], key);
                        insideObj[property].forEach((nestedObj) => {
                            firebaseDbFormatter(nestedObj, key);
                        });
                    }
                });
            }
        });
        const newObj = [];
        listOfIds.map(id => {
            newObj.push({ [key]: id, ...obj[id] });
        });
        return newObj;
    }
    return obj;
}
